// copyright (2019.) Marko Čupić - Fakultet elektrotehnike i računarstva, Sveučilište u Zagrebu; nije dozvoljena uporaba u vlastitim projektima, prerada ili daljna distribucija bez dozvole autora

function vector(arr) {
  this.elems = arr;
}

vector.prototype.add = function(other) {
  if(this.elems.length != other.elems.length) throw "Incompatible vectors for addition.";
  let res = [];
  for(let r = 0; r < this.elems.length; r++) {
    res.push(this.elems[r]+other.elems[r]);
  }
  return new vector(res);
}

vector.prototype.sub = function(other) {
  if(this.elems.length != other.elems.length) throw "Incompatible vectors for subtraction.";
  let res = [];
  for(let r = 0; r < this.elems.length; r++) {
    res.push(this.elems[r]-other.elems[r]);
  }
  return new vector(res);
}

vector.prototype.scalarProduct = function(other) {
  if(this.elems.length != other.elems.length) throw "Incompatible vectors for scalar product.";
  let res = 0;
  for(let r = 0; r < this.elems.length; r++) {
    res += this.elems[r]*other.elems[r];
  }
  return res;
}

vector.prototype.scalarMultiply = function(s) {
  let res = [];
  for(let r = 0; r < this.elems.length; r++) {
    res.push(s*this.elems[r]);
  }
  return new vector(res);
}

vector.prototype.norm = function() {
  let res = 0;
  for(let r = 0; r < this.elems.length; r++) {
    res += this.elems[r]*this.elems[r];
  }
  return Math.sqrt(res);
}

vector.prototype.normalize = function() {
  return this.scalarMultiply(1.0/this.norm());
}

vector.prototype.toString = function() {
  let res = "[";
  for(let r = 0; r < this.elems.length; r++) {
    if(r>0) res += ", ";
    res += this.elems[r];
  }
  res += "]";
  return res;
}

vector.prototype.orthogonal2D = function() {
  if(this.elems.length != 2) throw "orthogonal2D called on "+this.elems.length+"-dimensional vector.";
  return new vector([this.elems[1], -this.elems[0]]);
}

